/*===========================================================================
 * Project         : Mandriva Linux
 * Module          : mountloop
 * File            : common.c
 * Version         : $Id: common.c 145743 2005-11-14 14:04:39Z tvignaud $
 * Author          : Frederic Lepied
 * Created On      : Thu Jun 20 12:09:33 2002
 *===========================================================================*/

static const char rcs_id[] = "$Id: common.c 145743 2005-11-14 14:04:39Z tvignaud $";
static const char compile_id[] = "$Compile: " __FILE__ " " __DATE__ " " __TIME__ " $";

/*
 **********************************************************************
 *
 * Included files
 *
 **********************************************************************
 */

#include "common.h"
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/ioctl.h>
#include <fcntl.h>

/*
 **********************************************************************
 *
 * Constants
 *
 **********************************************************************
 */

/*
 **********************************************************************
 *
 * Macros
 *
 **********************************************************************
 */

/*
 **********************************************************************
 *
 * New types
 *
 **********************************************************************
 */

/*========================================*/
/* extracted from util-linux sources      */
/*========================================*/

#define LO_NAME_SIZE	64
#define LO_KEY_SIZE	32
       
struct loop_info {
	int		lo_number;	/* ioctl r/o */
	dev_t		lo_device; 	/* ioctl r/o */
	unsigned long	lo_inode; 	/* ioctl r/o */
	dev_t		lo_rdevice; 	/* ioctl r/o */
	int		lo_offset;
	int		lo_encrypt_type;
	int		lo_encrypt_key_size; 	/* ioctl w/o */
	int		lo_flags;	/* ioctl r/o */
	char		lo_name[LO_NAME_SIZE];
	unsigned char	lo_encrypt_key[LO_KEY_SIZE]; /* ioctl w/o */
	unsigned long	lo_init[2];
	char		reserved[4];
};

#define LO_CRYPT_NONE	0
#define LO_CRYPT_XOR	1
#define LO_CRYPT_DES	2
#define LO_CRYPT_IDEA	3
#define MAX_LO_CRYPT	4

#define LOOP_SET_FD	0x4C00
#define LOOP_CLR_FD	0x4C01
#define LOOP_SET_STATUS	0x4C02
#define LOOP_GET_STATUS	0x4C03

/*========================================*/

/*
 **********************************************************************
 *
 * External functions
 *
 **********************************************************************
 */

/*
 **********************************************************************
 *
 * External variables
 *
 **********************************************************************
 */

/*
 **********************************************************************
 *
 * Global variables
 *
 **********************************************************************
 */



/*
 **********************************************************************
 *
 * Constants
 *
 **********************************************************************
 */

#define MAXARG 256
#define MAXLOOP 256

/*
 **********************************************************************
 *
 * mysystem: simplified system
 *
 **********************************************************************
 */
int
mysystem(char	**argv,
	 uid_t	newid)
{
  pid_t pid;
  char	**p = argv;
  
  fprintf(stderr, "%c ", (newid > 0) ? '$' : '#');

  while(*p) {
    fprintf(stderr, "%s ", *p);
    p++;
  }
  
  fprintf(stderr, "\n");
  
  pid = fork();
  
  if (pid == 0) {
    if (newid > 0) {

      if (setuid(newid)) {
	perror("setuid");
	exit(EXIT_FAILURE);
      }
      
      if (!setuid(0)) {
	perror("setuid root");
	exit(EXIT_FAILURE);
      }
    }
    
    execv(argv[0], argv);
    perror("execv");
    exit(EXIT_FAILURE);
  } else {

    if (pid > 0) {
      int status;
      
      waitpid(pid, &status, 0);
      
      if (WIFEXITED(status)) {
	return(WEXITSTATUS(status));
      } else{
	return EXIT_FAILURE;
      }
    } else{
      perror("fork");
      return EXIT_FAILURE;
    }
  }
}

/*
 **********************************************************************
 *
 * find_free_loop: find a loop device not bound
 *
 **********************************************************************
 */
int
find_free_loop() 
{
  int	loop;
  char	str[2048];
  char  *argv[3];
  static int once = 0;

  if (!once) {
    argv[0] = "/sbin/modprobe";
    argv[1] = "aes";
    argv[2] = NULL;  
    mysystem(argv, 0);
    
    argv[1] = "cryptoloop";
    mysystem(argv, 0);

    once = 1;
  }
  
  for(loop=0; loop<MAXLOOP; loop++) {
    int	fd;

    /* Try to guess which loop device will be available.
     * This is important in non interactive mode because we will
     * let the subprocess read from stdin and this can only be done
     * once. In the future we'll need to read the pass by ourself and
     * pipe it to losetup until it works.
     */
      
    if (snprintf(str, sizeof(str), "/dev/loop%d", loop) <= 0) {
      fprintf(stderr, "not enough space for arguments\n");
      exit(EXIT_FAILURE);
    }

    fd = open(str, O_RDONLY);

    if (fd < 0) {
      perror(str);
      continue;
    } else {
      struct loop_info	loopinfo;
      int ret;
      
      ret = ioctl(fd, LOOP_GET_STATUS, &loopinfo);
      
      close(fd);
      
      /* device is in use */
      if (ret == 0) {
	continue;
      }
      break;
    }
  }

  if (loop == MAXLOOP) {
    fprintf(stderr, "no loop device available\n");
    exit(EXIT_FAILURE);
  }
  
  return loop;
}

/*
 **********************************************************************
 *
 * fileuidp: check if a path is a normal file and belongs to uid
 *
 **********************************************************************
 */
void
fileuidp(const char	*path,
	 uid_t		uid)
{
  struct stat	s;
  
  if (stat(path, &s)) {
    perror(path);
    exit(EXIT_FAILURE);
  }
  
  if (!S_ISREG(s.st_mode)) {
    fprintf(stderr, "is not a file %s\n", path);
    exit(EXIT_FAILURE);
  }
  
  if (s.st_uid != uid) {
    fprintf(stderr, "%s is not owned by you\n", path);
    exit(EXIT_FAILURE);
  }
}

/*
 **********************************************************************
 *
 * diruidp: check if a path is a directory and belongs to uid
 *
 **********************************************************************
 */
void
diruidp(const char	*path,
	uid_t		uid)
{
  struct stat	s;
  
  if (stat(path, &s)) {
    perror(path);
    exit(EXIT_FAILURE);
  }
  
  if (!S_ISDIR(s.st_mode)) {
    fprintf(stderr, "is not a directory %s\n", path);
    exit(EXIT_FAILURE);
  }
  
  if (s.st_uid != uid) {
    fprintf(stderr, "%s is not owned by you\n", path);
    exit(EXIT_FAILURE);
  }
}

/*
 * Local variables:
 * mode: c
 * End:
 *
 * common.c ends here
 */
