/*===========================================================================
 * Project         : Mandriva Linux
 * Module          : mountloop
 * File            : encsetup.c
 * Version         : $Id: encsetup.c 145743 2005-11-14 14:04:39Z tvignaud $
 * Author          : Frederic Lepied
 * Created On      : Wed Jun  5 20:57:28 2002
 *===========================================================================*/

static const char rcs_id[] = "$Id: encsetup.c 145743 2005-11-14 14:04:39Z tvignaud $";
static const char compile_id[] = "$Compile: " __FILE__ " " __DATE__ " " __TIME__ " $";

/*
 **********************************************************************
 *
 * Included files
 *
 **********************************************************************
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <fcntl.h>
#include "common.h"

int
main(int argc, char * argv[])
{
  uid_t		ruid = getuid();
  uid_t		euid = geteuid();
  char		strloop[2048];
  char		*ftype = "ext2";
  int		loop = 0;
  char		*myargv[10];
  int		myargc = 0;
  
  if (argc != 3 && argc != 4) {
    fprintf(stderr, "usage: %s <encrypt type> <file> [<fs type>]\n", argv[0]);
    exit(EXIT_FAILURE);
  }
  
  if (argc == 4) {
    ftype = argv[3];
  }
  
  if (euid != 0) {
    fprintf(stderr,"This program must be run as root or have the SUID attribute set\n");
    exit(EXIT_FAILURE);
  }
  
  if (setreuid(0, 0) != 0) {
    perror("setreuid");
    exit(EXIT_FAILURE);
  }

  fileuidp(argv[2], ruid);

  loop = find_free_loop();

  myargv[myargc++] = "/sbin/losetup";

  /* if we are in non interactive mode, we need to pass -p 0 to losetup */
  if (!isatty(0)) {
    myargv[myargc++] = "-p";
    myargv[myargc++] = "0";
  }
  
  myargv[myargc++] = "-e";
  myargv[myargc++] = argv[1];
  
  if (snprintf(strloop, sizeof(strloop), "/dev/loop%d", loop) <= 0) {
    fprintf(stderr, "not enough space for arguments\n");
    exit(EXIT_FAILURE);
  }

  myargv[myargc++] = strloop;
  myargv[myargc++] = argv[2];
  myargv[myargc++] = NULL;
  myargc = 0;
  
  if (mysystem(myargv, 0)) {
    fprintf(stderr, "loop device unaccessible\n");
    exit(EXIT_FAILURE);
  }
  
  fprintf(stderr, "# chown %d %s\n", ruid, strloop);
  
  if (chown(strloop, ruid, -1) < 0) {
    perror("chown");
    exit(EXIT_FAILURE);
  }

  myargv[myargc++] = "/sbin/mkfs";
  myargv[myargc++] = "-t";
  myargv[myargc++] = ftype;
  myargv[myargc++] = strloop;
  myargv[myargc++] = NULL;
  myargc = 0;
  
  if (mysystem(myargv, ruid)) {
    exit(EXIT_FAILURE);
  }
  
  myargv[myargc++] = "/sbin/losetup";
  myargv[myargc++] = "-d";
  myargv[myargc++] = strloop;
  myargv[myargc++] = NULL;
  myargc = 0;

  if (mysystem(myargv, 0)) {
    exit(EXIT_FAILURE);
  }

  if (chown(strloop, 0, -1) < 0) {
    perror("chown");
    exit(EXIT_FAILURE);
  }

  return EXIT_SUCCESS;
}

/*
 * Local variables:
 * mode: c
 * End:
 *
 * encsetup.c ends here
 */
