/*===========================================================================
 * Project         : Mandriva Linux
 * Module          : mountloop
 * File            : mountloop.c
 * Version         : $Id: mountloop.c 145743 2005-11-14 14:04:39Z tvignaud $
 * Author          : Frederic Lepied
 * Created On      : Wed Jun  5 20:57:28 2002
 *===========================================================================*/

static const char rcs_id[] = "$Id: mountloop.c 145743 2005-11-14 14:04:39Z tvignaud $";
static const char compile_id[] = "$Compile: " __FILE__ " " __DATE__ " " __TIME__ " $";

/*
 **********************************************************************
 *
 * Included files
 *
 **********************************************************************
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <string.h>
#include "common.h"

int
main(int argc, char * argv[])
{
  uid_t		ruid = getuid();
  uid_t		euid = geteuid();
  char		*myargv[10];
  int		myargc = 0;
  char		str[2048];
  int		ret;
  int		loop;
  
  if (argc != 4 && argc != 5) {
    fprintf(stderr, "usage: %s <enc> <file> <dir> [<fs type>]\n", argv[0]);
    exit(EXIT_FAILURE);
  }
  
  if (euid != 0) {
      fprintf(stderr,"This program must be run as root or have the SUID attribute set\n");
      exit(EXIT_FAILURE);
  }
  
  if (setreuid(0, 0) != 0) {
    perror("setreuid");
    exit(EXIT_FAILURE);
  }

  fileuidp(argv[2], ruid);
  diruidp(argv[3], ruid);

  loop = find_free_loop();
  
  if (snprintf(str, sizeof(str), "/dev/loop%d", loop) <= 0) {
    fprintf(stderr, "not enough space for arguments\n");
    exit(EXIT_FAILURE);
  }

  fprintf(stderr, "# chown %d %s\n", ruid, str);

  if (chown(str, ruid, -1) < 0) {
    perror("chown");
    exit(EXIT_FAILURE);
  }

  if (argc == 4) {
    myargv[myargc++] = "/bin/mount";
    myargv[myargc++] = "-o";
 
    ret = snprintf(str, sizeof(str), "loop=/dev/loop%d,encrypted,encryption=%s", loop, argv[1]);
    
    myargv[myargc++] = str;

    /* if we are in non interactive mode, we need to pass -p 0 to mount */
    if (!isatty(0)) {
      myargv[myargc++] = "-p";
      myargv[myargc++] = "0";
    }

    myargv[myargc++] = argv[2];
    myargv[myargc++] = argv[3];
    myargv[myargc++] = NULL;
    
  } else {
    myargv[myargc++] = "/bin/mount";
    myargv[myargc++] = "-t";
    myargv[myargc++] = argv[4];
    myargv[myargc++] = "-o";
    
    ret = snprintf(str, sizeof(str), "loop=/dev/loop%d,encrypted,encryption=%s", loop, argv[1]);
    
    myargv[myargc++] = str;

    /* if we are in non interactive mode, we need to pass -p 0 to mount */
    if (!isatty(0)) {
      myargv[myargc++] = "-p";
      myargv[myargc++] = "0";
    }
    
    myargv[myargc++] = argv[2];
    myargv[myargc++] = argv[3];
    myargv[myargc++] = NULL;
  }
  
  if (ret <= 0) {
    fprintf(stderr, "not enough space for arguments\n");
    exit(EXIT_FAILURE);
  }
  
  return mysystem(myargv, 0);
}

/*
 * Local variables:
 * mode: c
 * End:
 *
 * mountloop.c ends here
 */
